@@ -87,54 +87,52 @@ def prods(orig, ran, items, limit):
             if r == 0:
                 yield o
             else:
-                idxs = [0]*r
-                ilen = 0
+                idxs = [0]*min(r, ic)
                 while True:
                     stop = False
+                    setit = True
                     isum = 0
+                    previ = idxs[0]
                     for i in idxs:
+                        if previ > i:
+                            setit = False
+                        previ = i
                         isum += i
-                    if isum > ic:
-                        break
+                    if isum >= ic*min(r, ic)-min(r, ic):
+                        stop = True
                     args = []
-                    g = _gen(items, limit)
-                    # TODO filter None
-                    for i in idxs:
-                        for ii in range(i):
-                            try:
-                                v = next(g)
-                            #TODO catch only iteration exception
-                            except:
-                                stop = True
-                                break
-                        if stop == True:
-                            args.append(v)
-                            break
-                        try:
+                    if setit:
+                        for i in idxs:
+                            # TODO filter None?
+                            # TODO optimize! refactor!
+                            g = _gen(items, limit)
                             v = next(g)
+                            for ii in range(i):
+                                try:
+                                    v = next(g)
+                                except StopIteration:
+                                    try:
+                                        g = _gen(items, limit)
+                                        v = next(g)
+                                    except StopIteration:
+                                        stop = True
+                                        break
                             args.append(v)
-                        #TODO catch only iteration exception
-                        except:
-                            stop = True
-                            break
-                    if len(args) < ilen:
-                        break
-                    else:
-                        ilen = len(args)
+                        args = list(set(args))
                     nextflag = True
                     for k,i in enumerate(reversed(idxs)):
                         #reversed key
-                        k = r-k-1
+                        k = min(r,ic)-k-1
                         if nextflag:
                             idxs[k] += 1
-                        if idxs[k] > ic:
+                        if idxs[k] >= ic:
                             idxs[k] = 0
                             nextflag = True
                         else:
                             nextflag = False
-                    if not len(args):
-                        continue
                     for s in product(args, repeat=r):
+                        if len(s) and all(s[0] == x for x in s) and len(args) != 1:
+                            continue
                         yield o+''.join(s)
                     '''
                     print args
